import os
from dotenv import load_dotenv			# Import OS variables from Dotenv file.
load_dotenv()							# Load Dotenv. Delete this for production
import asyncio							# Discord Py Dependency
import discord							# Main Lib
from discord.ext import commands		# Commands module
from discord_slash import SlashCommand, SlashContext, cog_ext, utils			# Slash Command Library
from discord_slash.utils.manage_commands import create_choice, create_option	# Slash Command features
from pprint import pprint

from bot import clearConfig, configFile, checkConfig, loadCog, loadCogs, setConfig, unloadCog, unloadCogs, yaml_dump, yaml_load, reloadCog, reloadCogs, pitchesFile, cogsDir, parseConfigCheck

##### Debug Cog
class Debug(commands.Cog, name='Debug Commands'):
	def __init__(self, client):
		self.client = client

	#### Permission Check: Only available to the bot's maintainer.

	async def cog_check(self, ctx:commands.Context):
		conf = yaml_load(configFile)
		for role in ctx.author.roles:
			if 'maintainer' in conf[str(ctx.guild.id)]['roles']: 
				if role.id == conf[str(ctx.guild.id)]['roles']['maintainer']: return True
		return ctx.author.id == ctx.guild.owner_id

	@commands.command(
			name='testconfig',
			description='Tests the completeness of the configuration values of the current guild by comparing it to a configuration blueprint.',
			brief='Tests config values for current guild.',
			aliases=['configtest']
		)
	async def _testconfig(self, ctx:commands.Context):
		checkConfig(ctx.guild)
		status, output = checkConfig(ctx.guild)
		conf = yaml_load(configFile)
		if not status:
			await ctx.reply(f"```The Bot's configurations are incomplete for the guild {ctx.guild.name}. Some limited functions will still be available, but most features cannot be used until the configurations are complete.\n{parseConfigCheck(output)}\nYou can set these configuration values using the `/config` command.```")
		elif status:
			await ctx.reply(f"```The Bot's configurations for the guild {ctx.guild.name} are in order. The Bot is ready to interact with the guild.```")

	@commands.command(
		name='reloadcogs',
		description='Reloads cogs within the specified category, or provide `all` for all cogs. Default: `all`.',
		brief='Reload multiple cogs by category.'
	)
	async def _reload(self, ctx, cog_category: str='--all'):
		reloadCogs(cog_category)
		await ctx.reply(f'````{cog_category}` cogs have been reloaded.```')

	@commands.command(
		name='unloadcogs',
		description='Unload cogs within the specified category, or provide `all` for all cogs. Default: `all`.',
		brief='Unload multiple cogs by category.'
	)
	async def _unloadcogs(self, ctx, cog_category: str='--all'):
		unloadCogs(cog_category)
		loadCogs(cog_category)
		await ctx.reply(f'````{cog_category}` cogs have been unloaded.```')

	@commands.command(
		name='loadcogs',
		description='Load cogs within the specified category, or provide `all` for all cogs. Default: `all`.',
		brief='Load multiple cogs by category.'
	)
	async def _loadcogs(self, ctx, cog_category: str='--all'):
		unloadCogs(cog_category)
		loadCogs(cog_category)
		await ctx.reply(f'````{cog_category}` cogs have been loaded.```')

	@commands.command(
		name='retrievecommands',
		aliases=['slashcommands','retrieveslashcommands'],
		description='Debugging command that retrieves all slash commands currently registered for this guild and this bot to the Python console.',
		brief='Retrieves registered slash commands to console.'
	)
	async def _retrievecommands(self, ctx:commands.Context):
		c = await utils.manage_commands.get_all_commands(
				bot_id=self.client.user.id,
				bot_token=os.getenv('BOT_TOKEN'),
				guild_id=ctx.guild.id
			)
		pprint(c)
		await ctx.reply(f'```All registered `/commands` have been fetched and sent to the Python console.```')
	
	@commands.command(
		name='deletecommand',
		aliases=['removecommand','delcommand','removeslashcommand', 'clearcommand', 'clearslashcommand'],
		description='Debugging command that deletes a specified slash command. Key parameters `--all` for all commands in guild and `--global` for all commands globally',
		brief='Deletes slash command. Default: all local commands.'
	)
	async def _deleteCommand(self, ctx:commands.Context, command: str='--all'):
		if command == '--all' or command == '-a':
			await utils.manage_commands.remove_all_commands(
				bot_id=self.client.user.id,
				bot_token=os.getenv('BOT_TOKEN'),
				guild_ids=[ ctx.guild.id ]
			)
			await ctx.reply(f'```All slash commands have been deleted for the guild `{ctx.guild.name}``.```')
		elif command == '--global' or command == '-g':
			await utils.manage_commands.remove_all_commands(
				bot_id=self.client.user.id,
				bot_token=os.getenv('BOT_TOKEN'),
				guild_ids=None
			)
			await utils.manage_commands.remove_all_commands(
				bot_id=self.client.user.id,
				bot_token=os.getenv('BOT_TOKEN'),
				guild_ids=[ int(g) for g in yaml_load(configFile)]
			)
			await ctx.reply('```All slash commands have been deleted globally.```')
		else:
			c = await utils.manage_commands.get_all_commands(
				bot_id=self.client.user.id,
				bot_token=os.getenv('BOT_TOKEN'),
				guild_id=ctx.guild.id
			)
			target = list(filter(lambda t: t['name'] == command, c))[0]['id']
			await utils.manage_commands.remove_slash_command(
				bot_id=self.client.user.id,
				bot_token=os.getenv('BOT_TOKEN'),
				guild_id=ctx.guild.id,
				cmd_id=target
			)
			await ctx.reply(f'```Slash command {command} has been deleted for the guild {ctx.guild.name}.```')

	@commands.command(
		name='addcommand',
		aliases=['installcommand','addslashcommand'],
		description='Adds a slash command to the guild. Use keyword `--global` to add command globally.',
		brief='Adds slash command'
	)
	async def _addCommand(self, ctx:commands.Context, command:str, key:str=''):
		await utils.manage_commands.add_slash_command(
			bot_id=self.client.user.id,
			bot_token=os.getenv('BOT_TOKEN'),
			guild_id= None if key == '--global' or key == '-g' else ctx.guild.id,
			cmd_name=command,
			description='No Description'
		)
		await ctx.reply(f'```The command /{command} has been added for the guild {ctx.guild.name}.```')

	@commands.command(
		name='clearconfig',
		aliases=['configclear'],
		description='Clears any redundant entries in the config file of guilds the bot is not in. Does not require any argument.',
		brief='Clears redundant entries from config file.'
	)
	async def _clearconfig(self, ctx:commands.Context):
		conf = yaml_load(configFile)
		for key in list(conf):
			clearConfig(key)
		await ctx.reply(f'```Config entries for unknown guilds have been cleared.```')
	
	@commands.command(
		name='setconfig',
		aliases=['configsetup'],
		description='Creates a config entry for the current guild, and if there are existing entries sets default values. Ignores any values that have already been set. Does not require any argument as it infers the guild from the context in which it was called.',
		brief='Sets config entry for the current guild.'
	)
	async def _setconfig(self, ctx:commands.Context):
		setConfig(ctx.guild)
		await ctx.reply(f'```Config entry has been added for guild `{ctx.guild.name}`.```')

	@commands.command(
		name='synccommands',
		aliases=['syncallcommands','syncslashcommands','resynccommands','sync','resync','syncall','resyncall'],
		description='Syncs all slash commands between the bot and the Server.',
		brief='Resyncs slash commands.'
	)
	async def _synccommands(self, ctx:commands.Context):
		await self.client.slash.sync_all_commands()
		await ctx.reply(f'```All slash commands have been synced with the Server.```')
	
	@commands.command(
		name='pitchreset',
		description='Debug feature that resets the pitches in case of any error. Clears pitch disables Pitch listeners.',
		brief='Reset running pitches.',
		aliases=['resetpitches', 'resetpitch']
	)
	async def _pitchreset(self, ctx:commands.Context):
		yaml_dump({}, pitchesFile)
		if self.client.get_cog('Pitch Listener') is not None:
			unloadCog(f'./{cogsDir}/events/secondary/pitch_listener.py')
		await ctx.reply('```Pitches have been hard reset.```')

def setup(client):
	client.add_cog(Debug(client))