import os								# OS Locations
import yaml								# YAML parser for Bot config files
import asyncio							# Discord Py Dependency
import discord							# Main Lib
from discord.ext import commands		# Commands module
from discord_slash import SlashCommand, SlashContext, cog_ext, utils			# Slash Command Library
from discord_slash.utils.manage_commands import create_choice, create_option	# Slash Command features
import logging
# logger and handler
from bot import configFile, yaml_load, categoriesFile, configFile, lookupFile

##### Membership Restriction Message Listener Cog
class RestrictionListener(commands.Cog, name='Membership Restriction Listener'):
	def __init__(self, client):
		self.client = client
	
	# Block non-verified user from posting messages.
	@commands.Cog.listener(name='on_message')
	async def _restriction_listener(self,message):
		conf = yaml_load(configFile)
		categories = yaml_load(categoriesFile)
		guildStr = str(message.guild.id)
		lookup = yaml_load(lookupFile)
		if not conf[guildStr].get('restrict',False): return
		if message.author.bot: return
		if str(message.channel.category) in categories[guildStr]: return
		if (set(message.author.roles) & set([message.guild.get_role(x) for x in conf[guildStr]['roles']['admin']]) or message.author == message.guild.owner): return
		if set(message.author.roles) & set([message.guild.get_role(x) for x in conf[guildStr]['membership']]): return
		if message.channel.overwrites_for(message.author).manage_channels: return
		signupChannel = discord.utils.find(lambda x: x.id == conf[guildStr]['channels']['signup'], message.guild.channels)
		await message.channel.send(f'```{message.author.display_name} does not have a verified membership of `{message.guild.name}`. Please submit your membership confirmation for verification in the {signupChannel.name} to ensure you have access to your game.```\n{message.author.mention} | {signupChannel.mention}')
		await message.channel.category.set_permissions(message.author, send_messages = False, connect = False, reason=f'Membership Restriction: {message.author.display_name} is not a verified member.')
		await message.delete()
	
	# Reinstate on verification
	@commands.Cog.listener(name='on_member_update')
	async def _reinstate_listener(self, before, after):
		if before.roles == after.roles: return
		if len(set(after.roles) - set(before.roles)) != 1: return
		[d] = list(set(after.roles) - set(before.roles))
		conf = yaml_load(configFile)
		categories = yaml_load(categoriesFile)
		guildStr = str(after.guild.id)
		if d.id not in conf[guildStr]['membership']: return
		lookup = yaml_load(lookupFile)
		if not set(after.roles) & set([after.guild.get_role(x) for x in conf[guildStr]['membership']]): return
		for game in list(set(after.roles) & set([after.guild.get_role(int(x)) for x in lookup[guildStr]])):
			c = discord.utils.get(after.guild.categories, id=lookup[guildStr][str(game.id)]['category'])
			if c is not None:
				if c.overwrites_for(after).send_messages is False: await c.set_permissions(after, overwrite = None, reason= f'Membership Restriction: {after.display_name} has been verified and reinstated.')

def setup(client):
	client.add_cog(RestrictionListener(client))